/*
Copyright 2010 Harteex Productions
License: Creative Commons Attribution 3.0 Unported
http://creativecommons.org/licenses/by/3.0/
*/
#include "achievements.h"

gfx_font* gameFontAchievement  = NULL;
sprite* achievementLocked      = NULL;
sprite* achievementUnlocked    = NULL;
sprite* achievementCheck       = NULL;

uint8_t achievementsTaken = 0;
uint32_t cumulativeScore = 0;
list curAchievements;

char* achievementNames[] = {"Extinction", "Clear!", "Not Bad", "Master", "Collector", "You Must Be Bored", "Speedy", "Addicted"};
char* achievementDescriptions[] = {"Clear all of the same\ncolor at once", "Clear the whole board",
                                    "Achieve a highscore of\n3000 or more", "Achieve a highscore of\n4000 or more",
									"Achieve a cumulative\nscore of 75,000", "Achieve a cumulative\nscore of 250,000",
									"Finish with at least\n2000 points in 30s", "Start the game 20 times"};
bool achievementSecret[] = {false, false, false, false, false, false, false, true};

uint16_t achievementPopupTicks = 0;



void drawAchievementPopup(uint16_t w, uint16_t h, uint16_t screenW, uint16_t screenH)
{
	if (achievementPopupTicks > 0)
	{
		uint8_t* t = (uint8_t*)listGetFirst(&curAchievements);
		uint8_t a = *t;

		if (achievementPopupTicks <= 12)
		{
			w = (achievementPopupTicks * 220) / 12;
			h = (achievementPopupTicks * 40) / 12;
		}
		if (achievementPopupTicks >= 168)
		{
			int16_t t = -(achievementPopupTicks - 168);
			t += 12;
			w = (t * 220) / 12;
			h = (t * 40) / 12;
		}

		sprite* image = achievementUnlocked;
		if ((achievementPopupTicks >= 40 && achievementPopupTicks < 80) ||
			(achievementPopupTicks >= 120 && achievementPopupTicks < 160))
		{
			image = achievementCheck;
		}

		uint16_t yStart = screenH - 50;

		uint16_t xStartLeft = (screenW - w) / 2;
		gfx_rect_fill_draw(xStartLeft, yStart - (h / 2), w, h, gfx_color_rgb(0x60, 0x60, 0x60));
		gfx_rect_draw(xStartLeft, yStart - (h / 2), w, h, gfx_color_rgb(0x40, 0x40, 0x40));

		if (achievementPopupTicks > 12 && achievementPopupTicks < 168)
		{
			uint8_t tempOffset = 40;
			sprite_draw(image, xStartLeft + (tempOffset - image->width) / 2, yStart - (image->height / 2), 0);
			gfx_font_print(xStartLeft + tempOffset, yStart - (h / 2) + 3, gameFontAchievement, "Achievement Unlocked");
			gfx_font_print(xStartLeft + tempOffset, yStart - (h / 2) + 21, gameFontAchievement, achievementNames[a]);
		}
	}
}

void unlockAchievement(uint8_t a)
{
	if (!isAchievementUnlocked(a))
	{
		uint8_t* t = (uint8_t*)malloc(1);
		*t = a;
		listInsertLast(&curAchievements, t);

		uint8_t mask = 0x01 << a;
		achievementsTaken |= mask;
	}
}

bool isAchievementUnlocked(uint8_t a)
{
	uint8_t mask = 0x01 << a;
	return (mask & achievementsTaken) > 0;
}

bool checkAchievements()
{
	if (achievementPopupTicks == 0)
	{
		if (curAchievements.count > 0)
		{
			achievementPopupTicks = 1;
			return true;
		}
	}
	else
	{
		achievementPopupTicks++;

		if (achievementPopupTicks >= 180)
		{
			achievementPopupTicks = 0;
			listDeleteFirst(&curAchievements);
		}
	}

	return false;
}
